@@ -19,15 +19,13 @@ import unittest
 import time
 import logging
 import platform
-import sys
-
-import urlgrabber.progress
 
 from tests import URLTEST_LOCAL_MEDIA
 from tests import utils
 
 from virtinst import Guest
 from virtinst import urlfetcher
+from virtinst import util
 from virtinst.urlfetcher import FedoraDistro
 from virtinst.urlfetcher import SuseDistro
 from virtinst.urlfetcher import DebianDistro
@@ -59,7 +57,7 @@ UBUNTU_URL = "http://us.archive.ubuntu.com/ubuntu/dists/%s/main/installer-%s"
 
 OLD_DEBIAN_URL = "http://archive.debian.org/debian/dists/%s/main/installer-%s/"
 DAILY_DEBIAN_URL = "http://d-i.debian.org/daily-images/%s/"
-DEBIAN_URL = "http://ftp.us.debian.org/debian/dists/%s/main/installer-%s/"
+DEBIAN_URL = "ftp://ftp.us.debian.org/debian/dists/%s/main/installer-%s/"
 
 MANDRIVA_URL = "ftp://mirror.cc.columbia.edu/pub/linux/mandriva/official/%s/%s"
 
@@ -109,10 +107,10 @@ _set_distro(FedoraDistro)
 _add(ARCHIVE_FEDORA_URL % ("14", "x86_64"), "fedora14",
      i686=ARCHIVE_FEDORA_URL % ("14", "i386"))
 # 2 Latest releases
-_add(OLD_FEDORA_URL % ("20", "x86_64"), "fedora20")
 _add(FEDORA_URL % ("21", "x86_64"), "fedora21")
+_add(FEDORA_URL % ("22", "x86_64"), "fedora22")
 # Any Dev release
-_add(DEVFEDORA_URL % ("22", "x86_64"), "fedora21", name="fedora22")
+_add(FEDORA_URL % ("23", "x86_64"), "fedora22", name="fedora23")
 
 
 _set_distro(CentOSDistro)
@@ -122,10 +120,10 @@ _add(OLD_CENTOS_URL % ("4.9", "x86_64"), name="centos-4.9")
 # One old centos 5
 _add(OLD_CENTOS_URL % ("5.0", "x86_64"), name="centos-5.0")
 # Latest centos 5 w/ i686
-_add(CENTOS_URL % ("5", "x86_64"), "rhel5.8", name="centos-5-latest",
+_add(CENTOS_URL % ("5", "x86_64"), "rhel5.11", name="centos-5-latest",
      i686=CENTOS_URL % ("5", "i386"))
 # Latest centos 6 w/ i686
-_add(CENTOS_URL % ("6", "x86_64"), "centos6.5", name="centos-6-latest",
+_add(CENTOS_URL % ("6", "x86_64"), "rhel6.6", name="centos-6-latest",
      i686=CENTOS_URL % ("6", "i386"))
 # Latest centos 7, but no i686 as of 2014-09-06
 _add(CENTOS_URL % ("7", "x86_64"), "centos7.0", name="centos-7-latest")
@@ -135,7 +133,7 @@ _set_distro(SLDistro)
 # scientific 5
 _add(OLD_SCIENTIFIC_URL % ("55", "x86_64"), "rhel5.5", name="sl-5latest")
 # Latest scientific 6
-_add(SCIENTIFIC_URL % ("6", "x86_64"), "rhel6.1", name="sl-6latest")
+_add(SCIENTIFIC_URL % ("6", "x86_64"), "rhel6.6", name="sl-6latest")
 
 
 _set_distro(SuseDistro)
@@ -147,6 +145,9 @@ _add(OLD_OPENSUSE_URL % ("11.4"), "opensuse11.4", hasbootiso=False)
 # Only keep i686 for the latest opensuse
 _add(OPENSUSE_URL % ("12.3"), "opensuse12.3",
      i686=OPENSUSE_URL % ("12.3"), hasbootiso=False, testshortcircuit=True)
+# Latest 13.x releases
+_add(OPENSUSE_URL % ("13.1"), "opensuse13.1", hasbootiso=False)
+_add(OPENSUSE_URL % ("13.2"), "opensuse13.2", hasbootiso=False)
 
 
 _set_distro(DebianDistro)
@@ -156,8 +157,8 @@ _add(OLD_DEBIAN_URL % ("lenny", "amd64"), "debian5", hasxen=False,
 _add(DEBIAN_URL % ("squeeze", "amd64"), "debian6")
 _add(DEBIAN_URL % ("wheezy", "amd64"), "debian7")
 # And daily builds, since we specially handle that URL
-_add(DAILY_DEBIAN_URL % ("amd64"), "debian7", name="debiandaily")
-_add(DAILY_DEBIAN_URL % ("arm64"), "debian7",
+_add(DAILY_DEBIAN_URL % ("amd64"), "debian8", name="debiandaily")
+_add(DAILY_DEBIAN_URL % ("arm64"), "debian8",
     name="debiandailyarm64", hasxen=False)
 
 
@@ -185,9 +186,7 @@ hvmguest.os.os_type = "hvm"
 xenguest = Guest(testconn)
 xenguest.os.os_type = "xen"
 
-meter = urlgrabber.progress.BaseMeter()
-if utils.get_debug():
-    meter = urlgrabber.progress.TextMeter(fo=sys.stdout)
+meter = util.make_meter(quiet=not utils.get_debug())
 
 
 def _storeForDistro(fetcher, guest):
@@ -233,9 +232,10 @@ def _testURL(fetcher, distname, arch, distroobj):
         # Make sure the stores are reporting correct distro name/variant
         if (s and distroobj.detectdistro and
             distroobj.detectdistro != s.os_variant):
-            raise AssertionError("Store distro/variant did not match "
-                "expected values: store=%s, found=%s expect=%s" %
-                (s, s.os_variant, distroobj.detectdistro))
+            raise AssertionError(
+                "Store distro/variant did not match expected values:\n"
+                "url=%s\nstore=%s\nfound=%s\nexpect=%s" %
+                (fetcher.location, s, s.os_variant, distroobj.detectdistro))
 
     # Do this only after the distro detection, since we actually need
     # to fetch files for that part
