@@ -22,21 +22,20 @@ import logging
 import threading
 import traceback
 
-# pylint: disable=E0611
 from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
-# pylint: enable=E0611
 
 import libvirt
-import urlgrabber
 
-from virtManager.baseclass import vmmGObjectUI
+import virtinst.progress
+
+from .baseclass import vmmGObjectUI
 
 
-class vmmMeter(urlgrabber.progress.BaseMeter):
+class vmmMeter(virtinst.progress.BaseMeter):
     def __init__(self, cb_pulse, cb_fraction, cb_done):
-        urlgrabber.progress.BaseMeter.__init__(self)
+        virtinst.progress.BaseMeter.__init__(self)
         self.started = False
 
         self._vmm_pulse = cb_pulse
@@ -62,7 +61,7 @@ class vmmMeter(urlgrabber.progress.BaseMeter):
             text = self.text
         else:
             text = self.basename
-        fread = urlgrabber.progress.format_number(amount_read)
+        fread = virtinst.progress.format_number(amount_read)
         if self.size is None:
             out = "    %5sB" % (fread)
             self._vmm_pulse(out, text)
@@ -76,7 +75,7 @@ class vmmMeter(urlgrabber.progress.BaseMeter):
             text = self.text
         else:
             text = self.basename
-        fread = urlgrabber.progress.format_number(amount_read)
+        fread = virtinst.progress.format_number(amount_read)
         if self.size is None:
             out = "    %5sB" % (fread)
             self._vmm_pulse(out, text)
@@ -191,7 +190,6 @@ class vmmAsyncJob(vmmGObjectUI):
         self._bg_thread = threading.Thread(target=cb_wrapper,
                                            args=[callback, self] + args)
         self._bg_thread.daemon = True
-        logging.debug("Creating async job for function cb=%s", callback)
 
         self.builder.connect_signals({
             "on_async_job_delete_event" : self._on_window_delete,
