@@ -42,7 +42,8 @@ from .storagepool import vmmStoragePool
 
 
 # debugging helper to turn off events
-_disable_libvirt_events = False
+# Can be enabled with virt-manager --test-no-events
+FORCE_DISABLE_EVENTS = False
 
 
 class _ObjectList(vmmGObject):
@@ -53,6 +54,7 @@ class _ObjectList(vmmGObject):
         vmmGObject.__init__(self)
 
         self._objects = []
+        self._blacklist = []
         self._lock = threading.Lock()
 
     def _cleanup(self):
@@ -68,6 +70,29 @@ class _ObjectList(vmmGObject):
         finally:
             self._lock.release()
 
+    def _blacklist_key(self, obj):
+        return str(obj.__class__) + obj.get_connkey()
+
+    def add_blacklist(self, obj):
+        """
+        Add an object to the blacklist. Basically a list of objects we
+        choose not to poll, because they threw an error at init time
+
+        :param obj: vmmLibvirtObject to blacklist
+        :returns: True if object added, False if object was already in list
+        """
+        if self.in_blacklist(obj):
+            return False
+        self._blacklist.append(self._blacklist_key(obj))
+        return True
+
+    def in_blacklist(self, obj):
+        """
+        :param obj: vmmLibvirtObject to check
+        :returns: True if object is in the blacklist
+        """
+        return self._blacklist_key(obj) in self._blacklist
+
     def remove(self, obj):
         """
         Remove an object from the list.
@@ -81,6 +106,10 @@ class _ObjectList(vmmGObject):
             # Identity check is sufficient here, since we should never be
             # asked to remove an object that wasn't at one point in the list.
             if obj not in self._objects:
+                if self.in_blacklist(obj):
+                    self._blacklist.remove(self._blacklist_key(obj))
+                    return True
+
                 return False
 
             self._objects.remove(obj)
@@ -140,6 +169,7 @@ class vmmConnection(vmmGObject):
     __gsignals__ = {
         "vm-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "vm-removed": (GObject.SignalFlags.RUN_FIRST, None, [str]),
+        "vm-renamed": (GObject.SignalFlags.RUN_FIRST, None, [str, str]),
         "net-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "net-removed": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "pool-added": (GObject.SignalFlags.RUN_FIRST, None, [str]),
@@ -187,8 +217,8 @@ class vmmConnection(vmmGObject):
 
         self._objects = _ObjectList()
 
-        self.record = []
-        self.hostinfo = None
+        self._stats = []
+        self._hostinfo = None
 
         self.add_gsettings_handle(
             self._on_config_pretty_name_changed(
@@ -218,6 +248,10 @@ class vmmConnection(vmmGObject):
                 label = "test (xen)"
             elif gtype == "hvm":
                 label = "test (hvm)"
+        elif domtype == "qemu":
+            label = "QEMU TCG"
+        elif domtype == "kvm":
+            label = "KVM"
 
         return label
 
@@ -271,20 +305,15 @@ class vmmConnection(vmmGObject):
         return self._backend.invalidate_caps()
     caps = property(lambda self: getattr(self, "_backend").caps)
 
-    def pretty_host_memory_size(self):
-        if not self._backend.is_open():
-            return ""
-        return util.pretty_mem(self.host_memory_size())
-
     def host_memory_size(self):
         if not self._backend.is_open():
             return 0
-        return self.hostinfo[1] * 1024
+        return self._hostinfo[1] * 1024
 
     def host_active_processor_count(self):
         if not self._backend.is_open():
             return 0
-        return self.hostinfo[2]
+        return self._hostinfo[2]
 
     def connect(self, name, callback, *args):
         handle_id = vmmGObject.connect(self, name, callback, *args)
@@ -360,8 +389,6 @@ class vmmConnection(vmmGObject):
         """
         if self._get_config_pretty_name():
             return self._get_config_pretty_name()
-        if self._backend.fake_name():
-            return self._backend.fake_name()
 
         pretty_map = {
             "esx"       : "ESX",
@@ -375,7 +402,7 @@ class vmmConnection(vmmGObject):
             "uml"       : "UML",
             "vbox"      : "VBox",
             "vmware"    : "VMWare",
-            "xen"       : "xen",
+            "xen"       : "Xen",
             "xenapi"    : "XenAPI",
         }
 
@@ -388,7 +415,7 @@ class vmmConnection(vmmGObject):
 
         if is_session:
             ret += " User session"
-        elif path != "/system":
+        elif (path and path != "/system" and os.path.basename(path)):
             # Used by test URIs to report what XML file they are using
             ret += " %s" % os.path.basename(path)
 
@@ -596,7 +623,15 @@ class vmmConnection(vmmGObject):
     def filter_nodedevs(self, devtype=None, devcap=None):
         retdevs = []
         for dev in self.list_nodedevs():
-            xmlobj = dev.get_xmlobj()
+            try:
+                xmlobj = dev.get_xmlobj()
+            except libvirt.libvirtError, e:
+                # Libvirt nodedev XML fetching can be busted
+                # https://bugzilla.redhat.com/show_bug.cgi?id=1225771
+                if e.get_error_code() != libvirt.VIR_ERR_NO_NODE_DEVICE:
+                    logging.debug("Error fetching nodedev XML", exc_info=True)
+                continue
+
             if devtype and xmlobj.device_type != devtype:
                 continue
 
@@ -647,10 +682,7 @@ class vmmConnection(vmmGObject):
     def define_interface(self, xml):
         return self._backend.interfaceDefineXML(xml, 0)
 
-    def rename_object(self, obj, origxml, newxml, oldname, newname):
-        ignore = oldname
-        ignore = newname
-
+    def rename_object(self, obj, origxml, newxml, oldconnkey):
         if obj.class_name() == "domain":
             define_cb = self.define_domain
         elif obj.class_name() == "pool":
@@ -688,6 +720,9 @@ class vmmConnection(vmmGObject):
                 # Reinsert handle into new obj
                 obj.change_name_backend(newobj)
 
+        if newobj and obj.class_name() == "domain":
+            self.emit("vm-renamed", oldconnkey, obj.get_connkey())
+
 
     #########################
     # Domain event handling #
@@ -743,8 +778,8 @@ class vmmConnection(vmmGObject):
             return
 
         try:
-            if _disable_libvirt_events:
-                raise RuntimeError("_disable_libvirt_events = True")
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
 
             self._domain_cb_ids.append(
                 self.get_backend().domainEventRegisterAny(
@@ -780,8 +815,8 @@ class vmmConnection(vmmGObject):
             "device added")
 
         try:
-            if _disable_libvirt_events:
-                raise RuntimeError("_disable_libvirt_events = True")
+            if FORCE_DISABLE_EVENTS:
+                raise RuntimeError("FORCE_DISABLE_EVENTS = True")
 
             eventid = getattr(libvirt, "VIR_NETWORK_EVENT_ID_LIFECYCLE", 0)
             self._network_cb_ids.append(
@@ -821,7 +856,7 @@ class vmmConnection(vmmGObject):
             self._network_cb_ids = []
 
         self._backend.close()
-        self.record = []
+        self._stats = []
 
         if self._init_object_event:
             self._init_object_event.clear()
@@ -835,6 +870,13 @@ class vmmConnection(vmmGObject):
     def _cleanup(self):
         self.close()
 
+        self._objects = None
+        self._backend.cb_fetch_all_guests = None
+        self._backend.cb_fetch_all_pools = None
+        self._backend.cb_fetch_all_nodedevs = None
+        self._backend.cb_fetch_all_vols = None
+        self._backend.cb_clear_cache = None
+
     def open(self):
         if not self.is_disconnected():
             return
@@ -961,13 +1003,17 @@ class vmmConnection(vmmGObject):
 
         for obj in gone_objects:
             class_name = obj.class_name()
+            try:
+                name = obj.get_name()
+            except:
+                name = str(obj)
 
             if not self._objects.remove(obj):
                 logging.debug("Requested removal of %s=%s, but it's "
-                    "not in our object list.", class_name, obj.get_name())
+                    "not in our object list.", class_name, name)
                 continue
 
-            logging.debug("%s=%s removed", class_name, obj.get_name())
+            logging.debug("%s=%s removed", class_name, name)
             if class_name == "domain":
                 self.emit("vm-removed", obj.get_connkey())
             elif class_name == "network":
@@ -980,13 +1026,19 @@ class vmmConnection(vmmGObject):
                 self.emit("nodedev-removed", obj.get_connkey())
             obj.cleanup()
 
-    def _new_object_cb(self, obj):
+    def _new_object_cb(self, obj, initialize_failed):
         if not self._backend.is_open():
             return
 
         try:
             class_name = obj.class_name()
 
+            if initialize_failed:
+                logging.debug("Blacklisting %s=%s", class_name, obj.get_name())
+                if self._objects.add_blacklist(obj) is False:
+                    logging.debug("Object already blacklisted?")
+                return
+
             if not self._objects.add(obj):
                 logging.debug("New %s=%s requested, but it's already tracked.",
                     class_name, obj.get_name())
@@ -1064,6 +1116,7 @@ class vmmConnection(vmmGObject):
 
             gone_objects.extend(gone)
             preexisting_objects.extend([o for o in master if o not in new])
+            new = [n for n in new if not self._objects.in_blacklist(n)]
             return new
 
         new_vms = _process_objects(self._update_vms(pollvm))
@@ -1123,7 +1176,7 @@ class vmmConnection(vmmGObject):
         if self.using_network_events and not force:
             pollnet = False
 
-        self.hostinfo = self._backend.getInfo()
+        self._hostinfo = self._backend.getInfo()
 
         gone_objects, preexisting_objects = self._poll(
             initial_poll, pollvm, pollnet, pollpool, polliface, pollnodedev)
@@ -1169,9 +1222,9 @@ class vmmConnection(vmmGObject):
 
         now = time.time()
         expected = self.config.get_stats_history_length()
-        current = len(self.record)
+        current = len(self._stats)
         if current > expected:
-            del self.record[expected:current]
+            del self._stats[expected:current]
 
         mem = 0
         cpuTime = 0
@@ -1199,8 +1252,8 @@ class vmmConnection(vmmGObject):
         pcentHostCpu = 0
         pcentMem = mem * 100.0 / self.host_memory_size()
 
-        if len(self.record) > 0:
-            prevTimestamp = self.record[0]["timestamp"]
+        if len(self._stats) > 0:
+            prevTimestamp = self._stats[0]["timestamp"]
             host_cpus = self.host_active_processor_count()
 
             pcentHostCpu = ((cpuTime) * 100.0 /
@@ -1224,7 +1277,7 @@ class vmmConnection(vmmGObject):
             "netMaxRate" : netMaxRate,
         }
 
-        self.record.insert(0, newStats)
+        self._stats.insert(0, newStats)
 
 
     def schedule_priority_tick(self, **kwargs):
@@ -1270,64 +1323,45 @@ class vmmConnection(vmmGObject):
     # Stats getter methods #
     ########################
 
-    def _vector_helper(self, record_name):
+    def _get_record_helper(self, record_name):
+        if len(self._stats) == 0:
+            return 0
+        return self._stats[0][record_name]
+
+    def _vector_helper(self, record_name, limit, ceil=100.0):
         vector = []
-        stats = self.record
-        for i in range(self.config.get_stats_history_length() + 1):
-            if i < len(stats):
-                vector.append(stats[i][record_name] / 100.0)
+        statslen = self.config.get_stats_history_length() + 1
+        if limit is not None:
+            statslen = min(statslen, limit)
+
+        for i in range(statslen):
+            if i < len(self._stats):
+                vector.append(self._stats[i][record_name] / ceil)
             else:
                 vector.append(0)
+
         return vector
 
-    def stats_memory_vector(self):
-        return self._vector_helper("memoryPercent")
-
-    def host_cpu_time_vector(self):
-        return self._vector_helper("cpuHostPercent")
-    guest_cpu_time_vector = host_cpu_time_vector
-
-    def host_cpu_time_vector_limit(self, limit):
-        cpudata = self.host_cpu_time_vector()
-        if len(cpudata) > limit:
-            cpudata = cpudata[0:limit]
-        return cpudata
-    guest_cpu_time_vector_limit = host_cpu_time_vector_limit
-
-    def disk_io_vector_limit(self, ignore):
-        return [0.0]
-    def network_traffic_vector_limit(self, ignore):
-        return [0.0]
-
-    def _get_record_helper(self, record_name):
-        if len(self.record) == 0:
-            return 0
-        return self.record[0][record_name]
+    def stats_memory_vector(self, limit=None):
+        return self._vector_helper("memoryPercent", limit)
+    def host_cpu_time_vector(self, limit=None):
+        return self._vector_helper("cpuHostPercent", limit)
 
     def stats_memory(self):
         return self._get_record_helper("memory")
-    def pretty_stats_memory(self):
-        return util.pretty_mem(self.stats_memory())
-
     def host_cpu_time_percentage(self):
         return self._get_record_helper("cpuHostPercent")
-    guest_cpu_time_percentage = host_cpu_time_percentage
-
-    def network_rx_rate(self):
-        return self._get_record_helper("netRxRate")
-    def network_tx_rate(self):
-        return self._get_record_helper("netTxRate")
+    def guest_cpu_time_percentage(self):
+        return self.host_cpu_time_percentage()
     def network_traffic_rate(self):
-        return self.network_tx_rate() + self.network_rx_rate()
+        return (self._get_record_helper("netRxRate") +
+                self._get_record_helper("netTxRate"))
+    def disk_io_rate(self):
+        return (self._get_record_helper("diskRdRate") +
+                self._get_record_helper("diskWrRate"))
+
     def network_traffic_max_rate(self):
         return self._get_record_helper("netMaxRate")
-
-    def disk_read_rate(self):
-        return self._get_record_helper("diskRdRate")
-    def disk_write_rate(self):
-        return self._get_record_helper("diskWrRate")
-    def disk_io_rate(self):
-        return self.disk_read_rate() + self.disk_write_rate()
     def disk_io_max_rate(self):
         return self._get_record_helper("diskMaxRate")
 
@@ -1342,6 +1376,13 @@ class vmmConnection(vmmGObject):
         self.config.set_conn_autoconnect(self.get_uri(), val)
 
     def set_config_pretty_name(self, value):
+        cfgname = self._get_config_pretty_name()
+        if value == cfgname:
+            return
+        if not cfgname and value == self.get_pretty_desc():
+            # Don't encode the default connection value into gconf right
+            # away, require the user to edit it first
+            return
         self.config.set_perconn(self.get_uri(), "/pretty-name", value)
     def _get_config_pretty_name(self):
         return self.config.get_perconn(self.get_uri(), "/pretty-name")
