@@ -226,8 +226,9 @@ class vmmCreateNetwork(vmmGObjectUI):
         if self.widget("net-forward-none").get_active():
             return [None, None]
 
-        name = uiutil.get_list_selection(self.widget("net-forward"), 1)
-        mode = uiutil.get_list_selection(self.widget("net-forward-mode"), 1)
+        name = uiutil.get_list_selection(self.widget("net-forward"), column=1)
+        mode = uiutil.get_list_selection(
+            self.widget("net-forward-mode"), column=1)
         return [name, mode]
 
     def get_config_routev4_network(self):
@@ -362,7 +363,7 @@ class vmmCreateNetwork(vmmGObjectUI):
 
         if not ip.is_private:
             res = self.err.yes_no(_("Check Network Address"),
-                    _("The network should normally use a private IPv4 "
+                    _("The network should normally use a private IPv6 "
                       "address. Use this non-private address anyway?"))
             if not res:
                 return False
@@ -751,7 +752,7 @@ class vmmCreateNetwork(vmmGObjectUI):
         try:
             net = self._build_xmlobj()
         except Exception, e:
-            self.err.show_err(_("Error generating network xml: %s" % str(e)))
+            self.err.show_err(_("Error generating network xml: %s") % str(e))
             return
 
         self.topwin.set_sensitive(False)
