@@ -24,6 +24,7 @@ from gi.repository import GObject
 from gi.repository import Gtk
 
 from virtinst import Interface
+from virtinst import util
 
 from . import uiutil
 from .asyncjob import vmmAsyncJob
@@ -301,11 +302,11 @@ class vmmHost(vmmGObjectUI):
             self.conn.schedule_priority_tick(polliface=True)
 
     def refresh_resources(self, ignore=None):
-        vm_memory = self.conn.pretty_stats_memory()
-        host_memory = self.conn.pretty_host_memory_size()
+        vm_memory = util.pretty_mem(self.conn.stats_memory())
+        host_memory = util.pretty_mem(self.conn.host_memory_size())
+
         cpu_vector = self.conn.host_cpu_time_vector()
         memory_vector = self.conn.stats_memory_vector()
-
         cpu_vector.reverse()
         memory_vector.reverse()
 
@@ -341,8 +342,8 @@ class vmmHost(vmmGObjectUI):
                 _("Libvirt connection does not support interface management."))
 
         if conn_active:
-            uiutil.select_list_row_by_number(self.widget("net-list"), 0)
-            uiutil.select_list_row_by_number(self.widget("interface-list"), 0)
+            uiutil.set_list_selection_by_number(self.widget("net-list"), 0)
+            uiutil.set_list_selection_by_number(self.widget("interface-list"), 0)
             return
 
         self.set_net_error_page(_("Connection not active."))
@@ -488,11 +489,8 @@ class vmmHost(vmmGObjectUI):
         self.enable_net_apply(EDIT_NET_AUTOSTART)
 
     def current_network(self):
-        connkey = uiutil.get_list_selection(self.widget("net-list"), 0)
-        try:
-            return connkey and self.conn.get_net(connkey)
-        except KeyError:
-            return None
+        connkey = uiutil.get_list_selection(self.widget("net-list"))
+        return connkey and self.conn.get_net(connkey)
 
     def refresh_network(self, net):
         connkey = net.get_connkey()
@@ -524,14 +522,6 @@ class vmmHost(vmmGObjectUI):
 
         try:
             net = self.conn.get_net(connkey)
-        except KeyError:
-            self.disable_net_apply()
-            return
-        except Exception, e:
-            logging.exception(e)
-            self.set_net_error_page(_("Error selecting network: %s") % e)
-
-        try:
             self.populate_net_state(net)
         except Exception, e:
             logging.exception(e)
@@ -711,7 +701,7 @@ class vmmHost(vmmGObjectUI):
         finally:
             net_list.set_model(model)
 
-        uiutil.select_list_row_by_value(net_list,
+        uiutil.set_list_selection(net_list,
             curnet and curnet.get_connkey() or None)
 
 
@@ -782,11 +772,8 @@ class vmmHost(vmmGObjectUI):
         self.refresh_interface(cp)
 
     def current_interface(self):
-        connkey = uiutil.get_list_selection(self.widget("interface-list"), 0)
-        try:
-            return connkey and self.conn.get_interface(connkey)
-        except KeyError:
-            return None
+        connkey = uiutil.get_list_selection(self.widget("interface-list"))
+        return connkey and self.conn.get_interface(connkey)
 
     def interface_apply(self):
         interface = self.current_interface()
@@ -794,7 +781,7 @@ class vmmHost(vmmGObjectUI):
             return
 
         newmode = uiutil.get_list_selection(
-            self.widget("interface-startmode"), 0)
+            self.widget("interface-startmode"))
 
         logging.debug("Applying changes for interface '%s'",
                       interface.get_name())
@@ -964,7 +951,7 @@ class vmmHost(vmmGObjectUI):
         finally:
             iface_list.set_model(model)
 
-        uiutil.select_list_row_by_value(iface_list,
+        uiutil.set_list_selection(iface_list,
             curiface and curiface.get_connkey() or None)
 
     def populate_interface_children(self):
