@@ -158,12 +158,20 @@ class _VirtualCharDevice(VirtualDevice):
             return self.type in users[propname]
         return hasattr(self, propname)
 
+    def set_defaults(self, guest):
+        ignore = guest
+        if not self.source_mode and self.supports_property("source_mode"):
+            self.source_mode = self.MODE_BIND
+
+
     def _set_host_helper(self, hostparam, portparam, val):
         def parse_host(val):
             host, ignore, port = (val or "").partition(":")
             return host or None, port or None
 
         host, port = parse_host(val)
+        if not host:
+            host = "127.0.0.1"
         if host:
             setattr(self, hostparam, host)
         if port:
@@ -179,16 +187,15 @@ class _VirtualCharDevice(VirtualDevice):
 
     _XML_PROP_ORDER = ["type", "_has_mode_bind", "_has_mode_connect",
                        "bind_host", "bind_port",
-                       "source_mode", "_source_path", "source_channel",
-                       "source_host", "source_port",
+                       "source_mode", "source_host", "source_port",
+                       "_source_path", "source_channel",
                        "target_type", "target_name"]
 
-    type = XMLProperty(
-                doc=_("Method used to expose character device in the host."),
-                xpath="./@type")
+    type = XMLProperty("./@type",
+        doc=_("Method used to expose character device in the host."))
 
     _tty = XMLProperty("./@tty")
-    _source_path = XMLProperty(xpath="./source/@path",
+    _source_path = XMLProperty("./source/@path",
         doc=_("Host input path to attach to the guest."))
 
     def _get_source_path(self):
@@ -200,67 +207,49 @@ class _VirtualCharDevice(VirtualDevice):
         self._source_path = val
     source_path = property(_get_source_path, _set_source_path)
 
-    source_channel = XMLProperty(xpath="./source/@channel",
+    source_channel = XMLProperty("./source/@channel",
                                  doc=_("Source channel name."))
 
-    def _get_default_source_mode(self):
-        if self.type == self.TYPE_UDP:
-            return self.MODE_CONNECT
-        if not self.supports_property("source_mode"):
-            return None
-        return self.MODE_BIND
-    def _make_sourcemode_xpath(self):
-        if self.type == self.TYPE_UDP:
-            return "./source[@mode='connect']/@mode"
-        return "./source/@mode"
-    source_mode = XMLProperty(name="char sourcemode",
-                              doc=_("Target connect/listen mode."),
-                              make_xpath_cb=_make_sourcemode_xpath,
-                              default_cb=_get_default_source_mode)
 
-    def _get_default_sourcehost(self):
-        if not self.supports_property("source_host"):
-            return None
-        return "127.0.0.1"
-    def _set_source_validate(self, val):
-        if val is None or self.type != self.TYPE_UDP:
-            return val
-        if not self._has_mode_connect:
-            self._has_mode_connect = self.MODE_CONNECT
-        return val
-    def _make_sourcehost_xpath(self):
-        mode = self.source_mode
-        if self.type == self.TYPE_UDP:
-            mode = "connect"
-        return "./source[@mode='%s']/@host" % mode
-    source_host = XMLProperty(name="char sourcehost",
-                              doc=_("Address to connect/listen to."),
-                              make_xpath_cb=_make_sourcehost_xpath,
-                              default_cb=_get_default_sourcehost,
-                              set_converter=_set_source_validate)
+    ###################
+    # source handling #
+    ###################
 
-    def _make_sourceport_xpath(self):
-        return "./source[@mode='%s']/@service" % self.source_mode
-    source_port = XMLProperty(name="char sourceport",
-                        doc=_("Port on target host to connect/listen to."),
-                        make_xpath_cb=_make_sourceport_xpath,
-                        set_converter=_set_source_validate, is_int=True)
+    source_mode = XMLProperty("./source/@mode")
 
     _has_mode_connect = XMLProperty("./source[@mode='connect']/@mode")
     _has_mode_bind = XMLProperty("./source[@mode='bind']/@mode")
 
+    def _set_source_validate(self, val):
+        if val is None:
+            return None
+        self._has_mode_connect = self.MODE_CONNECT
+        return val
+    source_host = XMLProperty("./source[@mode='connect']/@host",
+                            doc=_("Host address to connect to."),
+                            set_converter=_set_source_validate)
+    source_port = XMLProperty("./source[@mode='connect']/@service",
+                              doc=_("Host port to connect to."),
+                              set_converter=_set_source_validate,
+                              is_int=True)
+
     def _set_bind_validate(self, val):
         if val is None:
             return None
-        if not self._has_mode_bind:
-            self._has_mode_bind = self.MODE_BIND
+        self._has_mode_bind = self.MODE_BIND
         return val
     bind_host = XMLProperty("./source[@mode='bind']/@host",
                             doc=_("Host address to bind to."),
                             set_converter=_set_bind_validate)
     bind_port = XMLProperty("./source[@mode='bind']/@service",
                             doc=_("Host port to bind to."),
-                            set_converter=_set_bind_validate, is_int=True)
+                            set_converter=_set_bind_validate,
+                            is_int=True)
+
+
+    #######################
+    # Remaining XML props #
+    #######################
 
     def _get_default_protocol(self):
         if not self.supports_property("protocol"):
