@@ -60,6 +60,8 @@ class VirtualPort(XMLBuilder):
     typeid = XMLProperty("./parameters/@typeid", is_int=True)
     typeidversion = XMLProperty("./parameters/@typeidversion", is_int=True)
     instanceid = XMLProperty("./parameters/@instanceid")
+    profileid = XMLProperty("./parameters/@profileid")
+    interfaceid = XMLProperty("./parameters/@interfaceid")
 
 
 class VirtualNetworkInterface(VirtualDevice):
@@ -95,7 +97,7 @@ class VirtualNetworkInterface(VirtualDevice):
         Generate a random MAC that doesn't conflict with any VMs on
         the connection.
         """
-        if hasattr(conn, "_virtinst__fake_conn_predictable"):
+        if conn.fake_conn_predictable():
             # Testing hack
             return "00:11:22:33:44:55"
 
@@ -173,7 +175,7 @@ class VirtualNetworkInterface(VirtualDevice):
 
     def _get_source(self):
         """
-        Convenince function, try to return the relevant <source> value
+        Convenience function, try to return the relevant <source> value
         per the network type.
         """
         if self.type == self.TYPE_VIRTUAL:
@@ -187,7 +189,7 @@ class VirtualNetworkInterface(VirtualDevice):
         return self._network or self._bridge or self._source_dev
     def _set_source(self, newsource):
         """
-        Conveninece function, try to set the relevant <source> value
+        Convenience function, try to set the relevant <source> value
         per the network type
         """
         self._bridge = None
@@ -230,6 +232,7 @@ class VirtualNetworkInterface(VirtualDevice):
     model = XMLProperty("./model/@type")
     target_dev = XMLProperty("./target/@dev")
     filterref = XMLProperty("./filterref/@filter")
+    link_state = XMLProperty("./link/@state")
 
     driver_name = XMLProperty("./driver/@name")
     driver_queues = XMLProperty("./driver/@queues", is_int=True)
