@@ -23,8 +23,6 @@ import shutil
 import subprocess
 import tempfile
 
-import urlgrabber
-
 from . import urlfetcher
 from . import util
 from .devicedisk import VirtualDisk
@@ -100,8 +98,7 @@ def _upload_file(conn, meter, destpool, src):
                 break
             data = data[ret:]
 
-    if meter is None:
-        meter = urlgrabber.progress.BaseMeter()
+    meter = util.ensure_meter(meter)
 
     # Build placeholder volume
     size = os.path.getsize(src)
@@ -311,6 +308,7 @@ class DistroInstaller(Installer):
         self.livecd = False
         self._cached_fetcher = None
         self._cached_store = None
+        self._cdrom_path = None
 
 
     ########################
@@ -331,8 +329,7 @@ class DistroInstaller(Installer):
         return MEDIA_LOCATION_CDROM
 
     def _get_fetcher(self, guest, meter):
-        if not meter:
-            meter = urlgrabber.progress.BaseMeter()
+        meter = util.ensure_meter(meter)
 
         if not self._cached_fetcher:
             scratchdir = util.make_scratchdir(guest.conn, guest.type)
@@ -350,16 +347,13 @@ class DistroInstaller(Installer):
         return self._cached_store
 
     def _prepare_local(self):
-        transient = True
-        if self.cdrom:
-            transient = not self.livecd
-        return self._make_cdrom_dev(self.location, transient=transient)
+        return self.location
 
     def _prepare_cdrom_url(self, guest, fetcher):
         store = self._get_store(guest, fetcher)
         media = store.acquireBootDisk(guest)
         self._tmpfiles.append(media)
-        return self._make_cdrom_dev(media, transient=True)
+        return media
 
     def _prepare_kernel_url(self, guest, fetcher):
         store = self._get_store(guest, fetcher)
@@ -418,12 +412,16 @@ class DistroInstaller(Installer):
             return _sanitize_url(val)
 
         try:
-            d = self._make_cdrom_dev(val)
-            val = d.path
-        except:
+            dev = VirtualDisk(self.conn)
+            dev.device = dev.DEVICE_CDROM
+            dev.path = val
+            dev.validate()
+
+            val = dev.path
+        except Exception, e:
             logging.debug("Error validating install location", exc_info=True)
-            raise ValueError(_("Checking installer location failed: "
-                               "Could not find media '%s'." % str(val)))
+            raise ValueError(_("Validating install media '%s' failed: %s") %
+                (str(val), e))
 
         return val
 
@@ -433,9 +431,9 @@ class DistroInstaller(Installer):
         if mediatype == MEDIA_CDROM_IMPLIED:
             return
 
-        dev = None
+        cdrom_path = None
         if mediatype == MEDIA_CDROM_PATH or mediatype == MEDIA_LOCATION_CDROM:
-            dev = self._prepare_local()
+            cdrom_path = self.location
 
         if mediatype != MEDIA_CDROM_PATH:
             fetcher = self._get_fetcher(guest, meter)
@@ -448,14 +446,13 @@ class DistroInstaller(Installer):
                     raise ValueError(_("Invalid install location: ") + str(e))
 
                 if mediatype == MEDIA_CDROM_URL:
-                    dev = self._prepare_cdrom_url(guest, fetcher)
+                    cdrom_path = self._prepare_cdrom_url(guest, fetcher)
                 else:
                     self._prepare_kernel_url(guest, fetcher)
             finally:
                 fetcher.cleanupLocation()
 
-        if dev:
-            self.install_devices.append(dev)
+        self._cdrom_path = cdrom_path
 
 
 
@@ -466,6 +463,14 @@ class DistroInstaller(Installer):
     def has_install_phase(self):
         return not self.livecd
 
+    def needs_cdrom(self):
+        mediatype = self._get_media_type()
+        return mediatype in [MEDIA_CDROM_PATH, MEDIA_LOCATION_CDROM,
+                             MEDIA_CDROM_URL]
+
+    def cdrom_path(self):
+        return self._cdrom_path
+
     def scratchdir_required(self):
         mediatype = self._get_media_type()
         return mediatype in [MEDIA_CDROM_URL, MEDIA_LOCATION_URL,
