@@ -20,8 +20,8 @@
 # MA 02110-1301 USA.
 
 import logging
+import os
 
-import urlgrabber.progress as progress
 import libvirt
 
 from virtcli import CLIConfig
@@ -35,6 +35,7 @@ from .device import VirtualDevice
 from .deviceaudio import VirtualAudio
 from .devicechar import VirtualChannelDevice, VirtualConsoleDevice
 from .devicecontroller import VirtualController
+from .devicedisk import VirtualDisk
 from .devicegraphics import VirtualGraphics
 from .deviceinput import VirtualInputDevice
 from .deviceredirdev import VirtualRedirDevice
@@ -102,7 +103,8 @@ class Guest(XMLBuilder):
     _XML_ROOT_NAME = "domain"
     _XML_PROP_ORDER = ["type", "name", "uuid", "title", "description",
         "maxmemory", "memory", "blkiotune", "memtune", "memoryBacking",
-        "vcpus", "curvcpus", "numatune", "bootloader", "os", "idmap",
+        "vcpus", "curvcpus", "vcpu_placement", "cpuset",
+        "numatune", "bootloader", "os", "idmap",
         "features", "cpu", "clock", "on_poweroff", "on_reboot", "on_crash",
         "resource", "pm", "emulator", "_devices", "seclabel"]
 
@@ -124,12 +126,15 @@ class Guest(XMLBuilder):
 
         self.__os_object = None
         self._random_uuid = None
-        self._install_devices = []
+        self._install_cdrom_device = None
         self._defaults_are_set = False
 
         # The libvirt virDomain object we 'Create'
         self.domain = None
 
+        # This is set via Capabilities.build_virtinst_guest
+        self.capsinfo = None
+
         self.installer = DistroInstaller(self.conn)
 
 
@@ -167,6 +172,7 @@ class Guest(XMLBuilder):
                         set_converter=_set_vcpus,
                         default_cb=lambda s: 1)
     curvcpus = XMLProperty("./vcpu/@current", is_int=True)
+    vcpu_placement = XMLProperty("./vcpu/@placement")
 
     def _validate_cpuset(self, val):
         DomainNumatune.validate_cpuset(self.conn, val)
@@ -192,11 +198,12 @@ class Guest(XMLBuilder):
                               default_cb=lambda s: "destroy")
     on_reboot = XMLProperty("./on_reboot", default_cb=lambda s: "restart")
     on_crash = XMLProperty("./on_crash", default_cb=lambda s: "restart")
+    on_lockfailure = XMLProperty("./on_lockfailure")
 
+    seclabel = XMLChildProperty(Seclabel)
     os = XMLChildProperty(OSXML, is_single=True)
     features = XMLChildProperty(DomainFeatures, is_single=True)
     clock = XMLChildProperty(Clock, is_single=True)
-    seclabel = XMLChildProperty(Seclabel, is_single=True)
     cpu = XMLChildProperty(CPU, is_single=True)
     numatune = XMLChildProperty(DomainNumatune, is_single=True)
     pm = XMLChildProperty(PM, is_single=True)
@@ -246,7 +253,7 @@ class Guest(XMLBuilder):
 
         @param dev: VirtualDevice instance to attach to guest
         """
-        self._add_child(dev)
+        self.add_child(dev)
 
     def remove_device(self, dev):
         """
@@ -254,7 +261,7 @@ class Guest(XMLBuilder):
 
         @param dev: VirtualDevice instance
         """
-        self._remove_child(dev)
+        self.remove_child(dev)
 
     def get_devices(self, devtype):
         """
@@ -269,6 +276,7 @@ class Guest(XMLBuilder):
             if devtype == "all" or i.virtual_device_type == devtype:
                 newlist.append(i)
         return newlist
+
     _devices = XMLChildProperty(
         [VirtualDevice.virtual_device_classes[_n]
          for _n in VirtualDevice.virtual_device_types],
@@ -289,18 +297,15 @@ class Guest(XMLBuilder):
     ############################
 
     def _prepare_install(self, meter, dry=False):
-        for dev in self._install_devices:
-            self.remove_device(dev)
-        self._install_devices = []
         ignore = dry
 
         # Fetch install media, prepare installer devices
         self.installer.prepare(self, meter)
 
         # Initialize install device list
-        for dev in self.installer.install_devices:
-            self.add_device(dev)
-            self._install_devices.append(dev)
+        if self._install_cdrom_device:
+            self._install_cdrom_device.path = self.installer.cdrom_path()
+            self._install_cdrom_device.validate()
 
     def _prepare_get_xml(self):
         # We do a shallow copy of the OS block here, so that we can
@@ -347,7 +352,8 @@ class Guest(XMLBuilder):
             return None
 
         self.installer.alter_bootconfig(self, osblob_install)
-        self._set_transient_device_defaults(install)
+        if not install:
+            self._remove_cdrom_install_media()
 
         if install:
             self.on_reboot = "destroy"
@@ -355,11 +361,6 @@ class Guest(XMLBuilder):
 
         self._set_osxml_defaults()
 
-        # Only set defaults on any install devices
-        for dev in self._install_devices:
-            dev.set_defaults(self)
-        self._set_disk_defaults(disks=self._install_devices)
-
         self.bootloader = None
         if (not install and
             self.os.is_xenpv() and
@@ -370,6 +371,86 @@ class Guest(XMLBuilder):
         return self.get_xml_config()
 
 
+    ###########################
+    # Private install helpers #
+    ###########################
+
+    def _build_meter(self, meter, is_initial):
+        if is_initial:
+            meter_label = _("Creating domain...")
+        else:
+            meter_label = _("Starting domain...")
+
+        meter = util.ensure_meter(meter)
+        meter.start(size=None, text=meter_label)
+
+        return meter
+
+    def _build_xml(self, is_initial):
+        log_label = is_initial and "install" or "continue"
+        disk_boot = not is_initial
+
+        start_xml = self._get_install_xml(install=True, disk_boot=disk_boot)
+        final_xml = self._get_install_xml(install=False)
+
+        logging.debug("Generated %s XML: %s",
+                      log_label,
+                      (start_xml and ("\n" + start_xml) or "None required"))
+        logging.debug("Generated boot XML: \n%s", final_xml)
+
+        return start_xml, final_xml
+
+    def _create_guest(self, meter,
+                      start_xml, final_xml, is_initial, noboot):
+        """
+        Actually do the XML logging, guest defining/creating
+
+        @param is_initial: If running initial guest creation, else we
+                           are continuing the install
+        @param noboot: Don't boot guest if no install phase
+        """
+        meter = self._build_meter(meter, is_initial)
+        doboot = not noboot or self.installer.has_install_phase()
+
+        if is_initial and doboot:
+            dom = self.conn.createLinux(start_xml or final_xml, 0)
+        else:
+            dom = self.conn.defineXML(start_xml or final_xml)
+            if doboot:
+                dom.create()
+
+        self.domain = dom
+        meter.end(0)
+
+        self.domain = self.conn.defineXML(final_xml)
+        if is_initial:
+            try:
+                logging.debug("XML fetched from libvirt object:\n%s",
+                              dom.XMLDesc(0))
+            except Exception, e:
+                logging.debug("Error fetching XML from libvirt object: %s", e)
+
+        return self.domain
+
+
+    def _flag_autostart(self):
+        """
+        Set the autostart flag for self.domain if the user requested it
+        """
+        if not self.autostart:
+            return
+
+        try:
+            self.domain.setAutostart(True)
+        except libvirt.libvirtError, e:
+            if util.is_error_nosupport(e):
+                logging.warn("Could not set autostart flag: libvirt "
+                             "connection does not support autostart.")
+            else:
+                raise e
+
+
+
     ##############
     # Public API #
     ##############
@@ -442,80 +523,36 @@ class Guest(XMLBuilder):
         return self._create_guest(meter,
                                   start_xml, final_xml, is_initial, False)
 
-    def _build_meter(self, meter, is_initial):
-        if is_initial:
-            meter_label = _("Creating domain...")
-        else:
-            meter_label = _("Starting domain...")
+    def get_created_disks(self):
+        return [d for d in self.get_devices("disk") if d.storage_was_created]
 
-        if meter is None:
-            meter = progress.BaseMeter()
-        meter.start(size=None, text=meter_label)
-
-        return meter
-
-    def _build_xml(self, is_initial):
-        log_label = is_initial and "install" or "continue"
-        disk_boot = not is_initial
-
-        start_xml = self._get_install_xml(install=True, disk_boot=disk_boot)
-        final_xml = self._get_install_xml(install=False)
-
-        logging.debug("Generated %s XML: %s",
-                      log_label,
-                      (start_xml and ("\n" + start_xml) or "None required"))
-        logging.debug("Generated boot XML: \n%s", final_xml)
-
-        return start_xml, final_xml
-
-    def _create_guest(self, meter,
-                      start_xml, final_xml, is_initial, noboot):
+    def cleanup_created_disks(self, meter):
         """
-        Actually do the XML logging, guest defining/creating
-
-        @param is_initial: If running initial guest creation, else we
-                           are continuing the install
-        @param noboot: Don't boot guest if no install phase
+        Remove any disks we created as part of the install. Only ever
+        called by clients.
         """
-        meter = self._build_meter(meter, is_initial)
-        doboot = not noboot or self.installer.has_install_phase()
-
-        if is_initial and doboot:
-            dom = self.conn.createLinux(start_xml or final_xml, 0)
-        else:
-            dom = self.conn.defineXML(start_xml or final_xml)
-            if doboot:
-                dom.create()
-
-        self.domain = dom
-        meter.end(0)
-
-        self.domain = self.conn.defineXML(final_xml)
-        if is_initial:
-            try:
-                logging.debug("XML fetched from libvirt object:\n%s",
-                              dom.XMLDesc(0))
-            except Exception, e:
-                logging.debug("Error fetching XML from libvirt object: %s", e)
-
-        return self.domain
-
-
-    def _flag_autostart(self):
-        """
-        Set the autostart flag for self.domain if the user requested it
-        """
-        if not self.autostart:
+        clean_disks = self.get_created_disks()
+        if not clean_disks:
             return
 
-        try:
-            self.domain.setAutostart(True)
-        except libvirt.libvirtError, e:
-            if util.is_error_nosupport(e):
-                logging.warn("Could not set autostart flag: libvirt "
-                             "connection does not support autostart.")
-            else:
-                raise e
+        for disk in clean_disks:
+            logging.debug("Removing created disk path=%s vol_object=%s",
+                disk.path, disk.get_vol_object())
+            name = os.path.basename(disk.path)
+
+            try:
+                meter.start(size=None, text=_("Removing disk '%s'") % name)
+
+                if disk.get_vol_object():
+                    disk.get_vol_object().delete()
+                else:
+                    os.unlink(disk.path)
+
+                meter.end(0)
+            except Exception, e:
+                logging.debug("Failed to remove disk '%s'",
+                    name, exc_info=True)
+                logging.error("Failed to remove disk '%s': %s", name, e)
 
 
     ###########################
@@ -588,20 +625,13 @@ class Guest(XMLBuilder):
     def add_default_console_device(self):
         if self.skip_default_console:
             return
-        if self.os.is_xenpv():
-            return
         if self.get_devices("console") or self.get_devices("serial"):
             return
 
         dev = VirtualConsoleDevice(self.conn)
         dev.type = dev.TYPE_PTY
-
-        if (self.os.is_x86() and
-            self._os_object.supports_virtioconsole() and
-            self.conn.check_support(
-            self.conn.SUPPORT_CONN_VIRTIO_CONSOLE)):
-            dev.target_type = "virtio"
-
+        if self.os.is_s390x():
+            dev.target_type = "sclp"
         self.add_device(dev)
 
     def add_default_video_device(self):
@@ -631,10 +661,11 @@ class Guest(XMLBuilder):
             return
         if self.get_devices("channel"):
             return
+        if self.os.is_s390x():
+            # Not wanted for s390 apparently
+            return
 
-        # Skip qemu-ga on ARM where virtio slots are currently limited
         if (self.conn.is_qemu() and
-            not self.os.is_arm() and
             self._os_object.supports_qemu_ga() and
             self.conn.check_support(self.conn.SUPPORT_CONN_AUTOSOCKET)):
             dev = VirtualChannelDevice(self.conn)
@@ -662,29 +693,31 @@ class Guest(XMLBuilder):
         self.add_default_usb_controller()
         self.add_default_channels()
 
-    def _set_transient_device_defaults(self, install):
-        def do_remove_media(d):
-            # Keep cdrom around, but with no media attached,
+    def _add_install_cdrom(self):
+        if self._install_cdrom_device:
+            return
+        if not self.installer.needs_cdrom():
+            return
+
+        dev = VirtualDisk(self.conn)
+        dev.device = dev.DEVICE_CDROM
+        setattr(dev, "installer_media", not self.installer.livecd)
+        self._install_cdrom_device = dev
+        self.add_device(dev)
+
+    def _remove_cdrom_install_media(self):
+        for dev in self.get_devices("disk"):
+            # Keep the install cdrom device around, but with no media attached.
             # But only if we are a distro that doesn't have a multi
             # stage install (aka not Windows)
-            return (d.is_cdrom() and
-                    getattr(d, "installer_media", False) and
-                    not install and
-                    not self.get_continue_inst())
-
-        def do_skip_disk(d):
-            # Skip transient labeled non-media disks
-            return (d.is_disk() and
-                    getattr(d, "installer_media", False) and
-                    not install)
-
-        for dev in self.get_devices("disk"):
-            if do_skip_disk(dev):
-                self.remove_device(dev)
-            elif do_remove_media(dev):
+            if (dev.is_cdrom() and
+                getattr(dev, "installer_media", False) and
+                not self.get_continue_inst()):
                 dev.path = None
 
     def _set_defaults(self):
+        self._add_install_cdrom()
+
         # some options check for has_spice() which is resolved after this:
         self._set_graphics_defaults()
 
@@ -765,9 +798,11 @@ class Guest(XMLBuilder):
         hpet.name = "hpet"
         hpet.present = False
 
-        if (self._os_object.is_windows() and
-            self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK) and
-            self._hv_supported()):
+        hv_clock = self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK)
+        hv_clock_rhel = self.conn.check_support(self.conn.SUPPORT_CONN_HYPERV_CLOCK_RHEL)
+
+        if (self._os_object.is_windows() and self._hv_supported() and
+            (hv_clock or (self.stable_defaults() and hv_clock_rhel))):
             hyperv = self.clock.add_timer()
             hyperv.name = "hypervclock"
             hyperv.present = True
@@ -806,7 +841,11 @@ class Guest(XMLBuilder):
         elif self.os.is_x86() and self.type == "kvm":
             if self.os.arch != self.conn.caps.host.cpu.arch:
                 return
+
             self.cpu.set_special_mode(self.x86_cpu_default)
+            if self._os_object.broken_x2apic():
+                self.cpu.add_feature("x2apic", policy="disable")
+
 
     def _hv_supported(self):
         if (self.os.loader_type == "pflash" and
@@ -814,6 +853,20 @@ class Guest(XMLBuilder):
             return False
         return True
 
+    def update_defaults(self):
+        # This is used only by virt-manager to reset any defaults that may have
+        # changed through manual intervention via the customize wizard.
+
+        # UEFI doesn't work with hyperv bits
+        if not self._hv_supported():
+            self.features.hyperv_relaxed = None
+            self.features.hyperv_vapic = None
+            self.features.hyperv_spinlocks = None
+            self.features.hyperv_spinlocks_retries = None
+            for i in self.clock.timers:
+                if i.name == "hypervclock":
+                    self.clock.remove_timer(i)
+
     def _set_feature_defaults(self):
         if self.os.is_container():
             self.features.acpi = None
@@ -832,13 +885,22 @@ class Guest(XMLBuilder):
             default = False
 
         if self.features.acpi == "default":
-            self.features.acpi = self._os_object.supports_acpi(default)
+            if default:
+                self.features.acpi = self.capsinfo.guest.supports_acpi()
+            else:
+                self.features.acpi = False
         if self.features.apic == "default":
-            self.features.apic = self._os_object.supports_apic(default)
+            self.features.apic = self.capsinfo.guest.supports_apic()
         if self.features.pae == "default":
-            self.features.pae = self.conn.caps.supports_pae()
+            if (self.os.is_hvm() and
+                self.type == "xen" and
+                self.os.arch == "x86_64"):
+                self.features.pae = True
+            else:
+                self.features.pae = self.capsinfo.guest.supports_pae()
 
         if (self.features.vmport == "default" and
+            self.os.is_x86() and
             self.has_spice() and
             self.conn.check_support(self.conn.SUPPORT_CONN_VMPORT)):
             self.features.vmport = False
@@ -949,10 +1011,8 @@ class Guest(XMLBuilder):
 
         return False
 
-    def _set_disk_defaults(self, disks=None):
-        alldisks = self.get_devices("disk")
-        if disks is None:
-            disks = alldisks
+    def _set_disk_defaults(self):
+        disks = self.get_devices("disk")
 
         def set_disk_bus(d):
             if d.is_floppy():
@@ -981,21 +1041,22 @@ class Guest(XMLBuilder):
                 d.bus = "sd"
             elif self.os.is_q35():
                 d.bus = "sata"
+            elif self.os.is_s390x():
+                d.bus = "virtio"
             else:
                 d.bus = "ide"
 
-
         # Generate disk targets
         used_targets = []
         for disk in disks:
             if not disk.bus:
                 set_disk_bus(disk)
 
-        for disk in alldisks:
-            if disk.target and not getattr(disk,
-                    "cli_generated_target", False):
+        for disk in disks:
+            if (disk.target and
+                not getattr(disk, "cli_generated_target", False)):
                 used_targets.append(disk.target)
-            elif disk in disks:
+            else:
                 disk.cli_generated_target = False
                 used_targets.append(disk.generate_target(used_targets))
 
@@ -1016,11 +1077,22 @@ class Guest(XMLBuilder):
                     net.model = net_model
 
     def _set_input_defaults(self):
+        def _usb_disabled():
+            controllers = [c for c in self.get_devices("controller") if
+                c.type == "usb"]
+            if not controllers:
+                return False
+            return all([c.model == "none" for c in controllers])
+
         input_type = self._os_object.default_inputtype()
         input_bus = self._os_object.default_inputbus()
         if self.os.is_xenpv():
             input_type = VirtualInputDevice.TYPE_MOUSE
             input_bus = VirtualInputDevice.BUS_XEN
+        elif _usb_disabled() and input_bus == "usb":
+            input_bus = "ps2"
+            if input_type == "tablet":
+                input_type = "mouse"
 
         for inp in self.get_devices("input"):
             if (inp.type == inp.TYPE_DEFAULT and
@@ -1047,8 +1119,9 @@ class Guest(XMLBuilder):
             gtype = self.default_graphics_type
             logging.debug("Using default_graphics=%s", gtype)
             if (gtype == "spice" and not
-                self.conn.check_support(
-                    self.conn.SUPPORT_CONN_GRAPHICS_SPICE)):
+                (self.conn.caps.host.cpu.arch in ["i686", "x86_64"] and
+                 self.conn.check_support(
+                     self.conn.SUPPORT_CONN_GRAPHICS_SPICE))):
                 logging.debug("spice requested but HV doesn't support it. "
                               "Using vnc.")
                 gtype = "vnc"
