@@ -19,32 +19,22 @@
 
 import logging
 
-import libvirt
-
-from virtinst.xmlbuilder import XMLBuilder
-from virtinst.xmlbuilder import XMLProperty as OrigXMLProperty
+from .xmlbuilder import XMLBuilder, XMLProperty
 
 
-# We had a pre-existing set of parse tests when this was converted to
-# XMLBuilder. We do this to appease the check in xmlparse.py without
-# moving all the nodedev.py tests to one file. Should find a way to
-# drop it.
-class XMLProperty(OrigXMLProperty):
-    def __init__(self, *args, **kwargs):
-        kwargs["track"] = False
-        OrigXMLProperty.__init__(self, *args, **kwargs)
+def _compare_int(nodedev_val, hostdev_val):
+    def _intify(val):
+        try:
+            if "0x" in str(val):
+                return int(val or '0x00', 16)
+            else:
+                return int(val)
+        except:
+            return -1
 
-
-def _lookupNodeName(conn, name):
-    try:
-        nodedev = conn.nodeDeviceLookupByName(name)
-    except libvirt.libvirtError, e:
-        raise libvirt.libvirtError(
-            _("Did not find node device '%s': %s" %
-            (name, str(e))))
-
-    xml = nodedev.XMLDesc(0)
-    return NodeDevice.parse(conn, xml)
+    nodedev_val = _intify(nodedev_val)
+    hostdev_val = _intify(hostdev_val)
+    return (nodedev_val == hostdev_val or hostdev_val == -1)
 
 
 class NodeDevice(XMLBuilder):
@@ -57,13 +47,8 @@ class NodeDevice(XMLBuilder):
     CAPABILITY_TYPE_SCSIBUS = "scsi_host"
     CAPABILITY_TYPE_SCSIDEV = "scsi"
 
-    (HOSTDEV_ADDR_TYPE_LIBVIRT,
-    HOSTDEV_ADDR_TYPE_PCI,
-    HOSTDEV_ADDR_TYPE_USB_BUSADDR,
-    HOSTDEV_ADDR_TYPE_USB_VENPRO) = range(1, 5)
-
     @staticmethod
-    def lookupNodeName(conn, name):
+    def lookupNodedevFromString(conn, idstring):
         """
         Convert the passed libvirt node device name to a NodeDevice
         instance, with proper error reporting. If the name is name is not
@@ -71,8 +56,11 @@ class NodeDevice(XMLBuilder):
         devAddressToNodeDev
 
         @param conn: libvirt.virConnect instance to perform the lookup on
-        @param name: libvirt node device name to lookup, or address for
-                     _devAddressToNodedev
+        @param idstring: libvirt node device name to lookup, or address
+            of the form:
+            - bus.addr (ex. 001.003 for a usb device)
+            - vendor:product (ex. 0x1234:0x5678 for a usb device
+            - (domain:)bus:slot.func (ex. 00:10.0 for a pci device)
 
         @rtype: L{NodeDevice} instance
         """
@@ -80,15 +68,19 @@ class NodeDevice(XMLBuilder):
             raise ValueError(_("Connection does not support host device "
                                "enumeration."))
 
+        # First try and see if this is a libvirt nodedev name
+        for nodedev in conn.fetch_all_nodedevs():
+            if nodedev.name == idstring:
+                return nodedev
+
         try:
-            return _lookupNodeName(conn, name)
-        except libvirt.libvirtError, e:
-            try:
-                _isAddressStr(name)
-            except:
-                raise e
+            return _AddressStringToNodedev(conn, idstring)
+        except Exception, e:
+            logging.debug("Error looking up nodedev from idstring=%s",
+                idstring, exc_info=True)
+            raise RuntimeError(_("Did not find node device matching '%s': %s" %
+                (idstring, e)))
 
-            return _devAddressToNodedev(conn, name)
 
     @staticmethod
     def parse(conn, xml):
@@ -101,12 +93,14 @@ class NodeDevice(XMLBuilder):
         if self.__class__ is NodeDevice and not instantiate:
             raise RuntimeError("Can not instantiate NodeDevice directly")
 
-        self.addr_type = None
-
         XMLBuilder.__init__(self, *args, **kwargs)
 
     _XML_ROOT_NAME = "device"
 
+    # Libvirt can generate bogus 'system' XML:
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1184131
+    _XML_SANITIZE = True
+
     name = XMLProperty("./name")
     parent = XMLProperty("./parent")
     device_type = XMLProperty("./capability/@type")
@@ -121,6 +115,10 @@ class NodeDevice(XMLBuilder):
         """
         return self.name
 
+    def compare_to_hostdev(self, hostdev):
+        ignore = hostdev
+        return False
+
 
 class SystemDevice(NodeDevice):
     hw_vendor = XMLProperty("./capability/hardware/vendor")
@@ -176,6 +174,15 @@ class PCIDevice(NodeDevice):
 
         return "%s %s %s" % (devstr, self.vendor_name, self.product_name)
 
+    def compare_to_hostdev(self, hostdev):
+        if hostdev.type != self.device_type:
+            return False
+
+        return (_compare_int(self.domain, hostdev.domain) and
+            _compare_int(self.bus, hostdev.bus) and
+            _compare_int(self.slot, hostdev.slot) and
+            _compare_int(self.function, hostdev.function))
+
 
 class USBDevice(NodeDevice):
     bus = XMLProperty("./capability/bus")
@@ -187,11 +194,50 @@ class USBDevice(NodeDevice):
     vendor_id = XMLProperty("./capability/vendor/@id")
 
     def pretty_name(self):
-        devstr = "%.3d:%.3d" % (int(self.bus), int(self.device))
-        desc = "%s %s %s" % (devstr, str(self.vendor_name),
-                             str(self.product_name))
+        # Hypervisor may return a rather sparse structure, missing
+        # some ol all stringular descriptions of the device altogether.
+        # Do our best to help user identify the device.
+
+        # Certain devices pad their vendor with trailing spaces,
+        # such as "LENOVO       ". It does not look well.
+        product = str(self.product_name).strip()
+        vendor = str(self.vendor_name).strip()
+
+        if product == "":
+            product = str(self.product_id)
+            if vendor == "":
+                # No stringular descriptions altogether
+                vendor = str(self.vendor_id)
+                devstr = "%s:%s" % (vendor, product)
+            else:
+                # Only the vendor is known
+                devstr = "%s %s" % (vendor, product)
+        else:
+            if vendor == "":
+                # Sometimes vendor is left out empty, but product is
+                # already descriptive enough or contains the vendor string:
+                # "Lenovo USB Laser Mouse"
+                devstr = product
+            else:
+                # We know everything. Perfect.
+                devstr = "%s %s" % (vendor, product)
+
+        busstr = "%.3d:%.3d" % (int(self.bus), int(self.device))
+        desc = "%s %s" % (busstr, devstr)
         return desc
 
+    def compare_to_hostdev(self, hostdev):
+        devtype = hostdev.type
+        if devtype == "usb":
+            devtype = "usb_device"
+        if devtype != self.device_type:
+            return False
+
+        return (_compare_int(self.product_id, hostdev.product) and
+            _compare_int(self.vendor_id, hostdev.vendor) and
+            _compare_int(self.bus, hostdev.bus) and
+            _compare_int(self.device, hostdev.device))
+
 
 class StorageDevice(NodeDevice):
     block = XMLProperty("./capability/block")
@@ -264,14 +310,13 @@ class SCSIBus(NodeDevice):
     wwpn = XMLProperty("./capability/capability[@type='fc_host']/wwpn")
 
 
-def _isAddressStr(addrstr):
-    cmp_func = None
-    addr_type = None
+def _AddressStringToHostdev(conn, addrstr):
+    from .devicehostdev import VirtualHostDevice
+    hostdev = VirtualHostDevice(conn)
 
     try:
         # Determine addrstr type
         if addrstr.count(":") in [1, 2] and addrstr.count("."):
-            devtype = NodeDevice.CAPABILITY_TYPE_PCI
             addrstr, func = addrstr.split(".", 1)
             addrstr, slot = addrstr.rsplit(":", 1)
             domain = "0"
@@ -280,83 +325,47 @@ def _isAddressStr(addrstr):
             else:
                 bus = addrstr
 
-            func = int(func, 16)
-            slot = int(slot, 16)
-            domain = int(domain, 16)
-            bus = int(bus, 16)
-
-            def pci_cmp(nodedev):
-                return ((int(nodedev.domain) == domain) and
-                        (int(nodedev.function) == func) and
-                        (int(nodedev.bus) == bus) and
-                        (int(nodedev.slot) == slot))
-            cmp_func = pci_cmp
-            addr_type = NodeDevice.HOSTDEV_ADDR_TYPE_PCI
+            hostdev.type = "pci"
+            hostdev.domain = "0x%.4X" % int(domain, 16)
+            hostdev.function = "0x%.2X" % int(func, 16)
+            hostdev.slot = "0x%.2X" % int(slot, 16)
+            hostdev.bus = "0x%.2X" % int(bus, 16)
 
         elif addrstr.count(":"):
-            devtype = NodeDevice.CAPABILITY_TYPE_USBDEV
             vendor, product = addrstr.split(":")
-            vendor = int(vendor, 16)
-            product = int(product, 16)
 
-            def usbprod_cmp(nodedev):
-                return ((int(nodedev.vendor_id, 16) == vendor) and
-                        (int(nodedev.product_id, 16) == product))
-            cmp_func = usbprod_cmp
-            addr_type = NodeDevice.HOSTDEV_ADDR_TYPE_USB_VENPRO
+            hostdev.type = "usb"
+            hostdev.vendor = "0x%.4X" % int(vendor, 16)
+            hostdev.product = "0x%.4X" % int(product, 16)
 
         elif addrstr.count("."):
-            devtype = NodeDevice.CAPABILITY_TYPE_USBDEV
-            bus, addr = addrstr.split(".", 1)
-            bus = int(bus)
-            addr = int(addr)
+            bus, device = addrstr.split(".", 1)
 
-            def usbaddr_cmp(nodedev):
-                return ((int(nodedev.bus) == bus) and
-                        (int(nodedev.device) == addr))
-            cmp_func = usbaddr_cmp
-            addr_type = NodeDevice.HOSTDEV_ADDR_TYPE_USB_BUSADDR
+            hostdev.type = "usb"
+            hostdev.bus = bus
+            hostdev.device = device
         else:
             raise RuntimeError("Unknown address type")
     except:
-        logging.exception("Error parsing node device string.")
+        logging.debug("Error parsing node device string.", exc_info=True)
         raise
 
-    return cmp_func, devtype, addr_type
+    return hostdev
 
 
-def _devAddressToNodedev(conn, addrstr):
-    """
-    Look up the passed host device address string as a libvirt node device,
-    parse its xml, and return a NodeDevice instance.
-
-    @param conn: libvirt.virConnect instance to perform the lookup on
-    @param addrstr: host device string to parse and lookup
-        - bus.addr (ex. 001.003 for a usb device)
-        - vendor:product (ex. 0x1234:0x5678 for a usb device
-        - (domain:)bus:slot.func (ex. 00:10.0 for a pci device)
-    @param addrstr: C{str}
-    """
-    try:
-        ret = _isAddressStr(addrstr)
-    except:
-        raise ValueError(_("Could not determine format of '%s'") % addrstr)
-
-    cmp_func, devtype, addr_type = ret
+def _AddressStringToNodedev(conn, addrstr):
+    hostdev = _AddressStringToHostdev(conn, addrstr)
 
     # Iterate over node devices and compare
     count = 0
     nodedev = None
 
-    nodenames = conn.listDevices(devtype, 0)
-    for name in nodenames:
-        tmpnode = _lookupNodeName(conn, name)
-        if cmp_func(tmpnode):
-            nodedev = tmpnode
+    for xmlobj in conn.fetch_all_nodedevs():
+        if xmlobj.compare_to_hostdev(hostdev):
+            nodedev = xmlobj
             count += 1
 
     if count == 1:
-        nodedev.addr_type = addr_type
         return nodedev
     elif count > 1:
         raise ValueError(_("%s corresponds to multiple node devices") %
