@@ -21,7 +21,8 @@ import logging
 
 
 # Debugging helper to force old style polling
-_force_old_poll = False
+# Can be enabled with virt-manager --test-old-poll
+FORCE_OLD_POLL = False
 
 
 def _new_poll_helper(origmap, typename, listfunc, buildfunc):
@@ -114,7 +115,7 @@ def fetch_nets(backend, origmap, build_func):
     name = "network"
 
     if backend.check_support(
-        backend.SUPPORT_CONN_LISTALLNETWORKS) and not _force_old_poll:
+        backend.SUPPORT_CONN_LISTALLNETWORKS) and not FORCE_OLD_POLL:
         return _new_poll_helper(origmap, name,
                                 backend.listAllNetworks, build_func)
     else:
@@ -131,7 +132,7 @@ def fetch_pools(backend, origmap, build_func):
     name = "pool"
 
     if backend.check_support(
-        backend.SUPPORT_CONN_LISTALLSTORAGEPOOLS) and not _force_old_poll:
+        backend.SUPPORT_CONN_LISTALLSTORAGEPOOLS) and not FORCE_OLD_POLL:
         return _new_poll_helper(origmap, name,
                                 backend.listAllStoragePools, build_func)
     else:
@@ -148,12 +149,13 @@ def fetch_volumes(backend, pool, origmap, build_func):
     name = "volume"
 
     if backend.check_support(
-        backend.SUPPORT_POOL_LISTALLVOLUMES, pool) and not _force_old_poll:
+        backend.SUPPORT_POOL_LISTALLVOLUMES, pool) and not FORCE_OLD_POLL:
         return _new_poll_helper(origmap, name,
                                 pool.listAllVolumes, build_func)
     else:
         active_list = pool.listVolumes
-        inactive_list = lambda: []
+        def inactive_list():
+            return []
         lookup_func = pool.storageVolLookupByName
         return _old_poll_helper(origmap, name,
                                 active_list, inactive_list,
@@ -164,7 +166,7 @@ def fetch_interfaces(backend, origmap, build_func):
     name = "interface"
 
     if backend.check_support(
-        backend.SUPPORT_CONN_LISTALLINTERFACES) and not _force_old_poll:
+        backend.SUPPORT_CONN_LISTALLINTERFACES) and not FORCE_OLD_POLL:
         return _new_poll_helper(origmap, name,
                                 backend.listAllInterfaces, build_func)
     else:
@@ -180,12 +182,14 @@ def fetch_interfaces(backend, origmap, build_func):
 def fetch_nodedevs(backend, origmap, build_func):
     name = "nodedev"
     if backend.check_support(
-        backend.SUPPORT_CONN_LISTALLDEVICES) and not _force_old_poll:
+        backend.SUPPORT_CONN_LISTALLDEVICES) and not FORCE_OLD_POLL:
         return _new_poll_helper(origmap, name,
                                 backend.listAllDevices, build_func)
     else:
-        active_list = lambda: backend.listDevices(None, 0)
-        inactive_list = lambda: []
+        def active_list():
+            return backend.listDevices(None, 0)
+        def inactive_list():
+            return []
         lookup_func = backend.nodeDeviceLookupByName
         return _old_poll_helper(origmap, name,
                                 active_list, inactive_list,
@@ -268,7 +272,7 @@ def _old_fetch_vms(backend, origmap, build_func):
             except:
                 logging.exception("Couldn't fetch domain '%s'", name)
 
-    return (origmap, new, current)
+    return (origmap.values(), new.values(), current.values())
 
 
 def fetch_vms(backend, origmap, build_func):
