@@ -23,13 +23,14 @@ import copy
 import logging
 import os
 import re
+import string  # pylint: disable=deprecated-module
 
 import libxml2
 
-from virtinst import util
+from . import util
 
 
-# pylint: disable=W0212
+# pylint: disable=protected-access
 # This whole file is calling around into non-public functions that we
 # don't want regular API users to touch
 
@@ -200,16 +201,18 @@ def _remove_xpath_node(ctx, xpath, dofree=True):
     """
     Remove an XML node tree if it has no content
     """
-    curxpath = xpath
+    nextxpath = xpath
     root_node = ctx.contextNode()
 
-    while curxpath:
+    while nextxpath:
+        curxpath = nextxpath
         is_orig = (curxpath == xpath)
         node = _get_xpath_node(ctx, curxpath)
+
         if curxpath.count("/"):
-            curxpath, ignore = curxpath.rsplit("/", 1)
+            nextxpath, ignore = curxpath.rsplit("/", 1)
         else:
-            curxpath = None
+            nextxpath = None
 
         if not node:
             continue
@@ -291,7 +294,7 @@ class XMLChildProperty(property):
             self._get(xmlbuilder).clear()
         else:
             for obj in self._get(xmlbuilder)[:]:
-                xmlbuilder._remove_child(obj)
+                xmlbuilder.remove_child(obj)
 
     def append(self, xmlbuilder, newobj):
         # Keep the list ordered by the order of passed in child classes
@@ -329,11 +332,10 @@ class XMLChildProperty(property):
 
 
 class XMLProperty(property):
-    def __init__(self, xpath=None, name=None, doc=None,
-                 set_converter=None, validate_cb=None, make_xpath_cb=None,
+    def __init__(self, xpath, doc=None,
+                 set_converter=None, validate_cb=None,
                  is_bool=False, is_int=False, is_yesno=False, is_onoff=False,
-                 clear_first=None, default_cb=None, default_name=None,
-                 track=True):
+                 default_cb=None, default_name=None, do_abspath=False):
         """
         Set a XMLBuilder class property that represents a value in the
         <domain> XML. For example
@@ -352,46 +354,36 @@ class XMLProperty(property):
             if xpath isn't specified.
         @param set_converter: optional function for converting the property
             value from the virtinst API to the guest XML. For example,
-            the Guest.memory API was once in MB, but the libvirt domain
-            memory API is in KB. So, if xpath is specified, on a 'get'
+            the Guest.memory API was once in MiB, but the libvirt domain
+            memory API is in KiB. So, if xpath is specified, on a 'get'
             operation we convert the XML value with int(val) / 1024.
         @param validate_cb: Called once when value is set, should
             raise a RuntimeError if the value is not proper.
-        @param make_xpath_cb: Not all props map cleanly to a
-            static xpath. This allows passing functions which generate
-            an xpath.
         @param is_bool: Whether this is a boolean property in the XML
         @param is_int: Whether this is an integer property in the XML
         @param is_yesno: Whether this is a yes/no property in the XML
         @param is_onoff: Whether this is an on/off property in the XML
-        @param clear_first: List of xpaths to unset before any 'set' operation.
-            For those weird interdependent XML props like disk source type and
-            path attribute.
         @param default_cb: If building XML from scratch, and this property
             is never explicitly altered, this function is called for setting
             a default value in the XML, and for any 'get' call before the
             first explicit 'set'.
         @param default_name: If the user does a set and passes in this
             value, instead use the value of default_cb()
-        @param track: If False, opt out of property tracking for the
-            test suite.
+        @param do_abspath: If True, run os.path.abspath on the passed value
         """
-
         self._xpath = xpath
-        self._name = name or xpath
-        if not self._name:
-            raise RuntimeError("XMLProperty: name or xpath must be passed.")
+        if not self._xpath:
+            raise RuntimeError("XMLProperty: xpath must be passed.")
         self._propname = None
 
         self._is_bool = is_bool
         self._is_int = is_int
         self._is_yesno = is_yesno
         self._is_onoff = is_onoff
+        self._do_abspath = do_abspath
 
-        self._make_xpath_cb = make_xpath_cb
         self._validate_cb = validate_cb
         self._convert_value_for_setter_cb = set_converter
-        self._setter_clear_these_first = clear_first or []
         self._default_cb = default_cb
         self._default_name = default_name
 
@@ -403,7 +395,8 @@ class XMLProperty(property):
         if self._default_name and not self._default_cb:
             raise RuntimeError("default_name requires default_cb.")
 
-        if _trackprops and track:
+        self._is_tracked = False
+        if _trackprops:
             _allprops.append(self)
 
         property.__init__(self, fget=self.getter, fset=self.setter)
@@ -411,7 +404,7 @@ class XMLProperty(property):
 
 
     def __repr__(self):
-        return "<XMLProperty %s %s>" % (str(self._name), id(self))
+        return "<XMLProperty %s %s>" % (str(self._xpath), id(self))
 
 
     ####################
@@ -433,12 +426,7 @@ class XMLProperty(property):
         return self._propname
 
     def _make_xpath(self, xmlbuilder):
-        ret = self._xpath
-        if self._make_xpath_cb:
-            ret = self._make_xpath_cb(xmlbuilder)
-        if ret is None:
-            raise RuntimeError("%s: didn't generate any xpath." % self)
-        return xmlbuilder.fix_relative_xpath(ret)
+        return xmlbuilder.fix_relative_xpath(self._xpath)
 
 
     def _build_node_list(self, xmlbuilder, xpath):
@@ -448,25 +436,6 @@ class XMLProperty(property):
         nodes = _get_xpath_node(xmlbuilder._xmlstate.xml_ctx, xpath)
         return util.listify(nodes)
 
-    def _build_clear_list(self, xmlbuilder, setternode):
-        """
-        Build a list of nodes that we should erase first before performing
-        a set operation. But we don't want to unset a node that we are
-        just going to 'set' on top of afterwards, so skip those ones.
-        """
-        clear_nodes = []
-
-        for cpath in self._setter_clear_these_first:
-            cpath = xmlbuilder.fix_relative_xpath(cpath)
-            cnode = _get_xpath_node(xmlbuilder._xmlstate.xml_ctx, cpath)
-            if not cnode:
-                continue
-            if setternode and setternode.nodePath() == cnode.nodePath():
-                continue
-            clear_nodes.append(cnode)
-        return clear_nodes
-
-
     def _convert_get_value(self, val):
         if self._default_name and val == self._default_name:
             ret = val
@@ -488,6 +457,8 @@ class XMLProperty(property):
     def _convert_set_value(self, xmlbuilder, val):
         if self._default_name and val == self._default_name:
             val = self._default_cb(xmlbuilder)
+        elif self._do_abspath and val is not None:
+            val = os.path.abspath(val)
         elif self._is_onoff and val is not None:
             val = bool(val) and "on" or "off"
         elif self._is_yesno and val is not None:
@@ -546,8 +517,6 @@ class XMLProperty(property):
         propstore = xmlbuilder._propstore
         proporder = xmlbuilder._proporder
 
-        if _trackprops and self not in _seenprops:
-            _seenprops.append(self)
         propname = self._findpropname(xmlbuilder)
         propstore[propname] = val
 
@@ -586,6 +555,10 @@ class XMLProperty(property):
         since it's known to the empty, and we may want to return
         a 'default' value
         """
+        if _trackprops and not self._is_tracked:
+            _seenprops.append(self)
+            self._is_tracked = True
+
         if (self._prop_is_unset(xmlbuilder) and
             not xmlbuilder._xmlstate.is_build):
             val = self._get_xml(xmlbuilder)
@@ -614,6 +587,10 @@ class XMLProperty(property):
         in propstore. Setting the actual XML is only done at
         get_xml_config time.
         """
+        if _trackprops and not self._is_tracked:
+            _seenprops.append(self)
+            self._is_tracked = True
+
         if validate and self._validate_cb:
             self._validate_cb(xmlbuilder, val)
         self._nonxml_fset(xmlbuilder,
@@ -630,27 +607,20 @@ class XMLProperty(property):
             ctx = _make_xml_context(root_node)
 
         xpath = self._make_xpath(xmlbuilder)
+
+        if setval is None or setval is False:
+            _remove_xpath_node(ctx, xpath)
+            return
+
         node = _get_xpath_node(xmlbuilder._xmlstate.xml_ctx, xpath)
-        clearlist = self._build_clear_list(xmlbuilder, node)
+        if not node:
+            node = _build_xpath_node(root_node, xpath)
 
-        node_map = []
-        if clearlist:
-            node_map += _tuplify_lists(clearlist, None,
-                                       [n.nodePath() for n in clearlist])
-        node_map += [(node, setval, xpath)]
+        if setval is True:
+            # Boolean property, creating the node is enough
+            return
 
-        for node, val, use_xpath in node_map:
-            if val is None or val is False:
-                _remove_xpath_node(ctx, use_xpath)
-                continue
-
-            if not node:
-                node = _build_xpath_node(root_node, use_xpath)
-
-            if val is True:
-                # Boolean property, creating the node is enough
-                continue
-            node.setContent(util.xml_escape(str(val)))
+        node.setContent(util.xml_escape(str(setval)))
 
 
 class _XMLState(object):
@@ -756,6 +726,14 @@ class XMLBuilder(object):
     # Name of the root XML element
     _XML_ROOT_NAME = None
 
+    # In some cases, libvirt can incorrectly generate unparseable XML.
+    # These are libvirt bugs, but this allows us to work around it in
+    # for specific XML classes.
+    #
+    # Example: nodedev 'system' XML:
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1184131
+    _XML_SANITIZE = False
+
     def __init__(self, conn, parsexml=None, parsexmlnode=None,
                  parent_xpath=None, relative_object_xpath=None):
         """
@@ -770,6 +748,10 @@ class XMLBuilder(object):
         """
         self.conn = conn
 
+        if self._XML_SANITIZE:
+            parsexml = parsexml.decode('ascii', 'ignore').encode('ascii')
+            parsexml = "".join([c for c in parsexml if c in string.printable])
+
         self._propstore = {}
         self._proporder = []
         self._xmlstate = _XMLState(self._XML_ROOT_NAME,
@@ -862,8 +844,20 @@ class XMLBuilder(object):
         for prop in props:
             prop.clear(self)
 
-        _remove_xpath_node(self._xmlstate.xml_ctx,
-                           self.get_root_xpath())
+        is_child = bool(re.match("^.*\[\d+\]$", self.get_root_xpath()))
+        if is_child:
+            # User requested to clear an object that is the child of
+            # another object (xpath ends in [1] etc). We can't fully remove
+            # the node in that case, since then the xmlbuilder object is
+            # no longer valid, and all the other child xpaths will be
+            # pointing to the wrong node. So just stub out the content
+            node = _get_xpath_node(self._xmlstate.xml_ctx,
+                                   self.get_root_xpath())
+            indent = 2 * self.get_root_xpath().count("/")
+            node.setContent("\n" + (indent * " "))
+        else:
+            _remove_xpath_node(self._xmlstate.xml_ctx,
+                               self.get_root_xpath())
 
     def validate(self):
         """
@@ -939,10 +933,10 @@ class XMLBuilder(object):
             for p in util.listify(getattr(self, propname, [])):
                 p._set_parent_xpath(self.get_root_xpath())
 
-    def _find_child_prop(self, child_class):
+    def _find_child_prop(self, child_class, return_single=False):
         xmlprops = self._all_child_props()
         for xmlprop in xmlprops.values():
-            if xmlprop.is_single:
+            if xmlprop.is_single and not return_single:
                 continue
             if child_class in xmlprop.child_classes:
                 return xmlprop
@@ -959,10 +953,11 @@ class XMLBuilder(object):
             for p in util.listify(getattr(self, propname, [])):
                 p._xmlstate._parse(None, self._xmlstate.xml_node)
 
-    def _add_child(self, obj):
+    def add_child(self, obj):
         """
         Insert the passed XMLBuilder object into our XML document. The
         object needs to have an associated mapping via XMLChildProperty
+        or an error is thrown.
         """
         xmlprop = self._find_child_prop(obj.__class__)
         xml = obj.get_xml_config()
@@ -976,10 +971,10 @@ class XMLBuilder(object):
             _build_xpath_node(self._xmlstate.xml_ctx, use_xpath, newnode)
         obj._parse_with_children(None, self._xmlstate.xml_node)
 
-    def _remove_child(self, obj):
+    def remove_child(self, obj):
         """
         Remove the passed XMLBuilder object from our XML document, but
-        ensure it's data isn't altered.
+        ensure its data isn't altered.
         """
         xmlprop = self._find_child_prop(obj.__class__)
         xmlprop.remove(self, obj)
@@ -992,6 +987,23 @@ class XMLBuilder(object):
         _remove_xpath_node(self._xmlstate.xml_ctx, xpath, dofree=False)
         self._set_child_xpaths()
 
+    def list_children_for_class(self, klass):
+        """
+        Return a list of all XML child objects with the passed class
+        """
+        ret = []
+        for prop in self._all_child_props().values():
+            ret += [obj for obj in util.listify(prop._get(self))
+                    if obj.__class__ == klass]
+        return ret
+
+    def child_class_is_singleton(self, klass):
+        """
+        Return True if the passed class is registered as a singleton
+        child property
+        """
+        return self._find_child_prop(klass, return_single=True).is_single
+
 
     #################################
     # Private XML building routines #
